@@ -24,14 +24,14 @@ static u_int16_t seq = 1;
 static u_int16_t ident = 0;
 
 static char *data;
-static size_t data_len = 0;
+static size_t *length_p;
 
 static int icmp_sk = -1;
 static int last_ttl = 0;
 
 
 static int icmp_init (const sockaddr_any *dest,
-				unsigned int port_seq, size_t packet_len) {
+			    unsigned int port_seq, size_t *packet_len_p) {
 	int i;
 	int af = dest->sa.sa_family;
 
@@ -40,11 +40,14 @@ static int icmp_init (const sockaddr_any *dest,
 
 	if (port_seq)  seq = port_seq;
 
-	data_len = sizeof (struct icmphdr) + packet_len;
-	data = malloc (data_len);
+	length_p = packet_len_p;
+	if (*length_p < sizeof (struct icmphdr))
+		*length_p = sizeof (struct icmphdr);
+
+	data = malloc (*length_p);
 	if (!data)  error ("malloc");
 
-        for (i = sizeof (struct icmphdr); i < data_len; i++)
+        for (i = sizeof (struct icmphdr); i < *length_p; i++)
                 data[i] = 0x40 + (i & 0x3f);
 
 
@@ -92,7 +95,7 @@ static void icmp_send_probe (probe *pb, int ttl) {
 	    icmp->icmp_id = htons (ident);
 	    icmp->icmp_seq = htons (seq);
 
-	    icmp->icmp_cksum = in_csum (data, data_len);
+	    icmp->icmp_cksum = in_csum (data, *length_p);
 	}
 	else if (af == AF_INET6) {
 	    struct icmp6_hdr *icmp6 = (struct icmp6_hdr *) data;
@@ -109,7 +112,7 @@ static void icmp_send_probe (probe *pb, int ttl) {
 
 	pb->send_time = get_time ();
 
-	if (do_send (icmp_sk, data, data_len, &dest_addr) < 0) {
+	if (do_send (icmp_sk, data, *length_p, &dest_addr) < 0) {
 	    pb->send_time = 0;
 	    return;
 	}
@@ -123,58 +126,20 @@ static void icmp_send_probe (probe *pb, int ttl) {
 }
 
 
-static void icmp_recv_probe (int sk, int revents) {
+static probe *icmp_check_reply (int sk, int err, sockaddr_any *from,
+						    char *buf, size_t len) {
 	int af = dest_addr.sa.sa_family;
-	struct msghdr msg;
-	sockaddr_any from;
-	struct iovec iov;
-	int n, type, err;
+	int type;
 	u_int16_t recv_id, recv_seq;
 	probe *pb;
-	char buf[1024];		/*  enough, enough...  */
-	char control[1024];
 
 
-	if (!(revents & (POLLIN | POLLERR)))
-		return;
-
-	err = !!(revents & POLLERR);
-
-
-	memset (&msg, 0, sizeof (msg));
-	msg.msg_name = &from;
-	msg.msg_namelen = sizeof (from);
-	msg.msg_control = control;
-	msg.msg_controllen = sizeof (control);
-	iov.iov_base = buf;
-	iov.iov_len = sizeof (buf);
-	msg.msg_iov = &iov;
-	msg.msg_iovlen = 1;
-
-
-	n = recvmsg (sk, &msg, err ? MSG_ERRQUEUE : 0);
-	if (n < sizeof (struct icmphdr))	/*  error or too short   */
-		return;
-
-	/*  for MSG_ERRQUEUE, an echoed original packet is returned
-	   (at least 8 bytes length)
-	*/
+	if (len < sizeof (struct icmphdr))
+		return NULL;
 
 
 	if (af == AF_INET) {
-	    struct icmp *icmp;
-
-	    if (!err) {
-		struct iphdr *ip = (struct iphdr *) buf;
-		int hlen = ip->ihl << 2;
-
-		if (n < hlen + sizeof (struct icmphdr))
-			return;
-
-		icmp = (struct icmp *) (buf + hlen);
-
-	    } else
-		icmp = (struct icmp *) buf;
+	    struct icmp *icmp = (struct icmp *) buf;
 
 	    type = icmp->icmp_type;
 
@@ -193,37 +158,37 @@ static void icmp_recv_probe (int sk, int revents) {
 
 
 	if (recv_id != ident)
-		return;
+		return NULL;
 
 	pb = probe_by_seq (recv_seq);
-	if (!pb)  return;
+	if (!pb)  return NULL;
 
 
 	if (!err) {
 
 	    if (!(af == AF_INET && type == ICMP_ECHOREPLY) &&
 		!(af == AF_INET6 && type == ICMP6_ECHO_REPLY)
-	    )  return;
-
-	    memcpy (&pb->res, &from, sizeof (pb->res));
+	    )  return NULL;
 
 	    pb->final = 1;
 	}
 
-	parse_cmsg (pb, &msg);	/*  err (if any), tstamp, ttl   */
+	return pb;
+}
 
 
-	pb->seq = -1;
+static void icmp_recv_probe (int sk, int revents) {
 
-	pb->done = 1;
+	if (!(revents & (POLLIN | POLLERR)))
+		return;
+
+	recv_reply (sk, !!(revents & POLLERR), icmp_check_reply);
 }
 
 
 static void icmp_expire_probe (probe *pb) {
 
-	pb->seq = -1;
-
-	pb->done = 1;
+	probe_done (pb);
 }
 
 
