@@ -85,6 +85,7 @@ static char **gateways = NULL;
 static int num_gateways = 0;
 static unsigned char *rtbuf = NULL;
 static size_t rtbuf_len = 0;
+static unsigned int ipv6_rthdr_type = 2;	/*  IPV6_RTHDR_TYPE_2   */
 
 static size_t header_len = 0;
 static size_t data_len = 0;
@@ -243,6 +244,20 @@ static void init_ip_options (void) {
 	if (!num_gateways)
 		return;
 
+	/*  check for TYPE,ADDR,ADDR... form   */
+	if (af == AF_INET6 && num_gateways > 1 && gateways[0]) {
+	    char *q;
+	    unsigned int value = strtoul (gateways[0], &q, 0);
+
+	    if (!*q) {
+		ipv6_rthdr_type = value;
+		num_gateways--;
+		for (i = 0; i < num_gateways; i++)
+			gateways[i] = gateways[i + 1];
+	    }
+	}
+
+
 	max = af == AF_INET ? MAX_GATEWAYS_4 : MAX_GATEWAYS_6;
 	if (num_gateways > max)
 	    ex_error ("Too many gateways specified. No more than %d", max);
@@ -298,7 +313,7 @@ static void init_ip_options (void) {
 	    rth = (struct ip6_rthdr *) rtbuf;
 	    rth->ip6r_nxt = 0;
 	    rth->ip6r_len = 2 * num_gateways;
-	    rth->ip6r_type = IPV6_RTHDR_TYPE_0;
+	    rth->ip6r_type = ipv6_rthdr_type;
 	    rth->ip6r_segleft = num_gateways;
 
 	    *((u_int32_t *) (rth + 1)) = 0;
@@ -428,7 +443,7 @@ static CLIF_option option_list[] = {
 			CLIF_set_flag, &dontfrag, 0, CLIF_ABBREV },
 	{ "f", "first", "first_ttl", "Start from the %s hop (instead from 1)",
 			CLIF_set_uint, &first_hop, 0, 0 },
-	{ "g", "gateway", "gate", "Route packets throw the specified gateway "
+	{ "g", "gateway", "gate", "Route packets through the specified gateway "
 			    "(maximum " _TEXT(MAX_GATEWAYS_4) " for IPv4 and "
 			    _TEXT(MAX_GATEWAYS_6) " for IPv6)",
 			add_gateway, 0, 0, CLIF_SEVERAL },
@@ -575,7 +590,7 @@ int main (int argc, char *argv[]) {
 
 	if (af == AF_INET6 && (tos || flow_label))
 		dst_addr.sin6.sin6_flowinfo =
-			((tos & 0xff) << 20) | (flow_label & 0x000fffff);
+		    htonl (((tos & 0xff) << 20) | (flow_label & 0x000fffff));
 
 	if (src_port) {
 	    src_addr.sin.sin_port = htons ((u_int16_t) src_port);
@@ -1043,7 +1058,7 @@ void tune_socket (int sk) {
 		flr.flr_label = htonl (flow_label & 0x000fffff);
                 flr.flr_action = IPV6_FL_A_GET;
                 flr.flr_flags = IPV6_FL_F_CREATE;
-                flr.flr_share = IPV6_FL_S_EXCL;
+                flr.flr_share = IPV6_FL_S_ANY;
 		memcpy (&flr.flr_dst, &dst_addr.sin6.sin6_addr,
 						    sizeof (flr.flr_dst));
 
@@ -1052,6 +1067,13 @@ void tune_socket (int sk) {
 		)  error ("setsockopt IPV6_FLOWLABEL_MGR");
 	    }
 
+	    if (tos) {
+		i = tos;
+		if (setsockopt (sk, IPPROTO_IPV6, IPV6_TCLASS,
+						    &i, sizeof (i)) < 0
+		)  error ("setsockopt IPV6_TCLASS");
+	    }
+
 	    if (tos || flow_label) {
 		i = 1;
 		if (setsockopt (sk, IPPROTO_IPV6, IPV6_FLOWINFO_SEND,
